/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * 
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * The Original Code is keywordManager.
 * 
 * The Initial Developer of the Original Code is 
 * Rgis Caspar. 
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 * 
 * Contributor(s): 
 *   Cory Schmidt: keywordBar author
 *   http://developer.mozilla.org/en/docs/Places:Query_System author(s)
 * 
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


var KWMView = null;


function Element(name, shortcut, bookmark)
{
  this.name = name;
  this.shortcut = shortcut;
  this.bookmark = bookmark;
}


function keywordmanager()
{
  this.elements = [];
  this.controllerLoaded = false;
  
  // *hack* to check if Places is enabled or not :)
  this.version = ('nsINavBookmarksService' in Components.interfaces) ? 2 : 1;
}


keywordmanager.prototype.init = function()
{
  var tree = document.getElementById("kwm-tree");
  
  switch (this.version)
  {
    case 1: // RDF bookmarks
      // ----------------------------------
      //   borrowed from keywordBar-0.3.1  
      // ----------------------------------
      var dataSource = Components.classes["@mozilla.org/rdf/datasource;1?name=bookmarks"]
                                 .getService(Components.interfaces.nsIRDFDataSource);
      var service = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                              .getService(Components.interfaces.nsIRDFService);
      var resources = dataSource.GetAllResources();
	var shortcutURLPredicate = service.GetResource( "http://home.netscape.com/NC-rdf#ShortcutURL" ); 
      var namePredicate = service.GetResource( "http://home.netscape.com/NC-rdf#Name" );
      
      while( resources.hasMoreElements( ) )
      {
        var bookmark = resources.getNext( );
      
        if( bookmark instanceof Components.interfaces.nsIRDFResource )
        {
          var name = dataSource.GetTarget( bookmark, namePredicate, true );
          var shortcutURL = dataSource.GetTarget( bookmark, shortcutURLPredicate, true );
         
          if( shortcutURL instanceof Components.interfaces.nsIRDFLiteral &&
              name instanceof Components.interfaces.nsIRDFLiteral )
          {
            this.elements.push( new Element(name.Value, shortcutURL.Value, 
                                              bookmark.Value) );
          }
        }
      }
      
      break;

    case 2: // Places
      // --------------------------------------------------------------------------
      //   borrowed from http://developer.mozilla.org/en/docs/Places:Query_System
      // --------------------------------------------------------------------------
      var historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"]
                                     .getService(Components.interfaces.nsINavHistoryService);
      var bookmarkService = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
                                      .getService(Components.interfaces.nsINavBookmarksService);
      // get all history sorted by ascending visit date
      var options = historyService.getNewQueryOptions();
      options.sortingMode = options.SORT_BY_TITLE_ASCENDING;
      // no query parameters will return everything
      var query = historyService.getNewQuery();
      query.setFolders([bookmarkService.bookmarksRoot], 1);
      query.onlyBookmarked = true;
      options.setGroupingMode([options.GROUP_BY_FOLDER], 1);
      var result = historyService.executeQuery(query, options);
      var cont = result.root;
      this.walk(cont, this.elements);
      
      break;
      
    default:
      // nothing
  }
  
  // update tree
  KWMView =  new keywordmanagerView(this.elements);
  tree.view = KWMView;
  KWMView.cycleHeader( { id:"kwm-treecol-name" } );
}


keywordmanager.prototype.walk  = function(node, elem)
{
  var node = node.QueryInterface(Components.interfaces.nsINavHistoryFolderResultNode);
  node.containerOpen = true;
  
  for (var i = 0; i < node.childCount; i ++) {
    var child = node.getChild(i);
  
    switch(child.type)
    {
      case node.RESULT_TYPE_FOLDER:
        this.walk(child, elem);
        break;
      case node.RESULT_TYPE_SEPARATOR:
        break;
      default:
        var bms = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
                            .getService(Components.interfaces.nsINavBookmarksService);
        var ios = Components.classes["@mozilla.org/network/io-service;1"]
                            .getService(Components.interfaces.nsIIOService);
        var uri = ios.newURI(child.uri, null, null);
        var kw = bms.getKeywordForURI(uri);
        if (kw)
          elem.push( new Element(child.title, kw, uri) );
    }
  }
  
  node.containerOpen = false;
}


keywordmanager.prototype.clearFilter = function()
{
  var fb = document.getElementById("kwm-filter-box");
  var but = document.getElementById("kwm-filter-button");
  var tree = document.getElementById("kwm-tree");
  
  but.setAttribute("disabled", "true");
  fb.value = ""; 
  fb.focus();
  
  // update tree
  KWMView = new keywordmanagerView(KWM.elements);
  tree.view = KWMView;
  KWMView.cycleHeader( { id:"kwm-treecol-name" } );
  KWMView.invalidate();
}


keywordmanager.prototype.setFilter = function(aTextBox)
{
  var but = document.getElementById("kwm-filter-button");
  var tree = document.getElementById("kwm-tree");
  
  if (aTextBox.value == "")
  {
    KWM.clearFilter();
    return;
  }
  
  var regexp = new RegExp(aTextBox.value, "i");
  var elements = [];
  
  but.setAttribute("disabled", "false");  
  elements = KWM.elements.filter(function(e, i, a) {
                                   return (e.name.match(regexp) ||
                                           e.shortcut.match(regexp));
                                 });
  
  // update tree
  KWMView = new keywordmanagerView(elements);
  tree.view = KWMView;
  KWMView.invalidate();
}


keywordmanager.prototype.destroy = function()
{
  KWM = null;
  KWMView = null;
}


keywordmanager.prototype.doubleclick = function(event)
{
  var tree = document.getElementById("kwm-tree");
  var row = {}, col = {}, obj = {};
  var name, shortcut;
  tree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, obj);
  
  if (obj.value == "text")
  {
    var elem = KWMView.selectedElement;
    var index = KWMView.selectedIndex;
    
    // -------------------------------------------------------------------
    //   borrowed from 'chrome://browser/content/bookmarks/bookmarks.js'
    // -------------------------------------------------------------------
    if (KWM.version == 1)
    {
      var value = {};
      openDialog( "chrome://browser/content/bookmarks/bookmarksProperties.xul", 
                  "", "centerscreen,chrome,modal,resizable=no", elem.bookmark, 
                  value );
      var service = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                              .getService(Components.interfaces.nsIRDFService);
      var resource = service.GetResource(elem.bookmark);
      var bmds = service.GetDataSource("rdf:bookmarks");
      var shortcutURLPredicate = service.GetResource( "http://home.netscape.com/NC-rdf#ShortcutURL" ); 
      var namePredicate = service.GetResource( "http://home.netscape.com/NC-rdf#Name" );
      var name = bmds.GetTarget(resource, namePredicate, true);
      var shortcut = bmds.GetTarget(resource, shortcutURLPredicate, true);
      name = name.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
      shortcut = shortcut.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
    } 
    else
    { 
      if (!KWM.controllerLoaded)
      {
        var loader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                              .getService(Components.interfaces.mozIJSSubScriptLoader);
        loader.loadSubScript("chrome://browser/content/places/controller.js");
        KWM.controllerLoaded = true;
      }
      
      var bms = Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"]
                            .getService(Components.interfaces.nsINavBookmarksService);
                            
      PlacesController.showBookmarkProperties(elem.bookmark);
      name = bms.getItemTitle(elem.bookmark);
      shortcut = bms.getKeywordForURI(elem.bookmark);
    }
    
    // update tree
    elem.name = name;
    elem.shortcut = shortcut;
    KWMView.invalidate();
  } 
}


var KWM = new keywordmanager();

